////////////////////////////////////////////////////////////////////////////////
// Images preloading

// Array to store preloaded images.
var preload_images_arr = new Array();

// Preload images so that there was no flickering when showing previously invisible
// (and not loaded by the browser) picture.
function preload_images(image_names) {
	for (var i=0; i<image_names.length; ++i) {
		preload_images_arr[i] = new Image;
		preload_images_arr[i].src = 'images/' + image_names[i];
	}
}


////////////////////////////////////////////////////////////////////////////////
// AJAX

// -----------------------------------------------------------------------------
// Global settings (overridable from different HTMLs).

// The progress dialog is not shown at once so that user was not annoyed by
// flickering (local requests are usually fast enough). But if it takes longer
// time for some reason, the dialog is displayed. Here the time in milliseconds
// is specified, after which the dialog will be shown, if AJAX request is still
// in progress
var timeout_show_progress = 250;

// -----------------------------------------------------------------------------
// AJAX internals.

// Global variable containing current AJAX request (several requests cannot be
// processed simultaneously in our script).
var ajax_req;

// Indicator, whether the request is in progress now.
var ajax_req_busy = false;

// Full URL to the AJAX server script. Must be on the same domain name, else
// AJAX does not work (security issue), even if two domain names point to the
// same real location.
var ajax_srv_url = document.location.protocol + '//' + document.location.host + '/ajax_srv.pl';

// Progress dialog.
var progressDiv = null;

// Error message area. Can be absent - in this case AJAX errors will be shown in
// a message box.
var err_area = null;
var err_text = null;
var ajax_error = '';

// AJAX initialization. Must be started after the HTML document is loaded.
function ajaxInit() {
	progressDiv = document.getElementById('ajax_loading');
	if (!progressDiv)
		ajax_error += 'Cannot find essential HTML entries!\n';
	else {
		if (navigator.appName == 'Microsoft Internet Explorer')
			progressDiv.style.height = '7.5em';
	}
	err_area = document.getElementById('error_msg_area');
	err_text = document.getElementById('error_msg_text');
	if (typeof(ajaxProcessResult) == 'undefined')
		ajax_error += 'No ajaxProcessResult was found!\n';
}

// Displaying AJAX error message, depending on whether HTML provides special
// location for them or not.
function ajaxError(error_text) {
	if (err_text && err_area) {
		err_text.innerHTML = error_text;
		err_area.style.display = '';
	}
	else
		alert(error_text.replace(/<br\s*\/>/g, '\n'));
}

// Starts counting time before the progress dialog appears.
function setBusyOn() {
	ajax_req_busy = true;
	setTimeout('showAjaxProgress()', timeout_show_progress);
}

// Displays the progress dialog.
function showAjaxProgress() {
	if (ajax_req_busy)
		progressDiv.style.display = '';
}

// Hides the progress dialog and marks AJAX request as not running.
function setBusyOff() {
	ajax_req_busy = false;
	progressDiv.style.display = 'none';
}

// Forces aborting the current AJAX call.
function onAbort() {
	setBusyOff();
	ajax_req.abort();
}

// Initiates the AJAX request (GET type).
function ajax_call_get(get_params) {
	if (ajax_error) {
		alert('Internal errors:\n' + ajax_error + '\nAJAX will not work correctly. Please, write to linux@ispras.ru.');
		return;
	}
	if (!ajax_req_busy) {
		setBusyOn();
		loadXMLDoc(ajax_srv_url + '?' + get_params, 'GET', null);
	}
}

// Initiates the AJAX request (POST type).
// form_id is the form where to take parameters from.
function ajax_call_post(get_params, form_id) {
	if (ajax_error) {
		alert('Internal errors:\n' + ajax_error + '\nAJAX will not work correctly. Please, write to linux@ispras.ru.');
		return;
	}
	if (!ajax_req_busy) {
		setBusyOn();
		loadXMLDoc(ajax_srv_url + '?' + get_params, 'POST', getFormValues(document.getElementById(form_id)));
	}
}

// Simulates the POST request by collecting all the fields from the form and
// combining them into POST data.
function getFormValues(fobj)
{
	var str = '';
	for(var i=0; i<fobj.elements.length; ++i)
	{
		if ((fobj.elements[i].type == 'hidden') || (fobj.elements[i].type == 'text') ||
			(fobj.elements[i].type == 'textarea') || (fobj.elements[i].type == 'select-one'))
		{
			if (fobj.elements[i].value)
				str += fobj.elements[i].name + '=' + encodeURIComponent(fobj.elements[i].value) + '&';
		}
	}
	str = str.substr(0, (str.length - 1));
	return str;
}

// Creates a cross-browser AJAX request object and performs the data transfer.
function loadXMLDoc(url, method, post_params) {
	if (window.XMLHttpRequest) {        // native XMLHttpRequest object
		ajax_req = new XMLHttpRequest();
	}
	else if (window.ActiveXObject) {    // IE/Windows ActiveX version
		try {
			ajax_req = new ActiveXObject('Msxml2.XMLHTTP');
		}
		catch (e) {
			try {
				ajax_req = new ActiveXObject('Microsoft.XMLHTTP');
			}
			catch (e) {
				alert('Sorry, your browser does not support AJAX.');
				return false;
			}
		}
	}
	if (!ajax_req) {
		alert('Sorry, your browser does not support AJAX.');
		return false;
	}

    ajax_req.onreadystatechange = processReqChange;
    ajax_req.open(method, url, true);
	ajax_req.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	// Workaround for IE always caching AJAX results:
	ajax_req.setRequestHeader('If-Modified-Since', 'Tue, 01 Jan 1980 00:00:00 GMT');
	ajax_req.setRequestHeader('Content-length', (post_params ? post_params.length : 0));
	ajax_req.setRequestHeader('Connection', 'close');
    ajax_req.send(post_params);
}

// Main function for processing results.
function processReqChange() {
	if (ajax_req.readyState == 4) {     // Process only 'Finished' state.
		setBusyOff();
        if (ajax_req.status == 200) {
			if (ajax_req.responseXML.getElementsByTagName('error').length > 0) {
	            var error_text = ajax_req.responseXML.getElementsByTagName('error')[0].childNodes[0].nodeValue;
            	ajaxError(error_text);
			}
			else {
				if (err_area)
            		err_area.style.display = 'none';
            	ajaxProcessResult(ajax_req.responseXML);
			}
        }
        else {
            alert('AJAX call failed:\n' + ajax_req.statusText);
        }
    }
}
